class Categories:

  @staticmethod
  def get():
    return {
      'text_greetings': 0,
      'language_translation': 1,
      'creative_writing': 2,
      'data_analysis': 3,
      'code_python': 4,
      'math_equation': 5,
      'code_generation': 6,
      'code_javascript': 7,
      'text_poetry': 8,
      'nonfiction_research': 9,
      'customer_service': 10,
      'technology_exploration': 11,
      'history_search': 12,
      'technology_analysis': 13,
      'philosophical_question': 14,
      'code_java': 15,
      'code_c++': 16,
      'technology_tutorial': 17,
      'technology_support': 18,
      'computer_programming': 19,
      'software_development': 20,
      'code_c#': 21,
      'history_analysis': 22,
      'health_advice': 23,
      'content_creation': 24,
      'psychology_support': 25,
      'math_problem': 26,
      'business_analysis': 27,
      'linguistic_analysis': 28,
      'technology_review': 29,
      'place_search': 30,
      'text_analysis': 31,
      'science_study': 32,
      'scientific_explanation': 33,
      'fact_checking': 34,
      'science_exploration': 35,
      'historical_analysis': 36,
      'international_law': 37,
      'science_theory': 38,
      'science_solution': 39,
      'code_php': 40,
      'finance_analysis': 41,
      'finance_advice': 42,
      'psychology_theory': 43,
      'science_tutorial': 44,
      'code_sql': 45,
      'science_introduction': 46,
      'business_management': 47,
      'place_analysis': 48,
      'education_tutorial': 49,
      'business_creation': 50,
      'biographies_of_figures': 51,
      'travel_advice': 52,
      'web_design': 53,
      'business_support': 54,
      'math_analysis': 55,
      'customer_engagement': 56,
      'translation_teaching': 57,
      'philosophy_theory': 58,
      'marketing_support': 59,
      'philosophy_question': 60,
      'gaming_analysis': 61,
      'philosophy_analysis': 62,
      'philosophy_exploration': 63
    }